@@ -10,6 +10,7 @@ import deskbar.Match
 
 from deskbar.defs import VERSION
 from deskbar.Watcher import FileWatcher
+from deskbar.Utils import spawn_async
 
 
 HANDLERS = {
@@ -34,7 +35,12 @@ class FileMatch(deskbar.Match.Match):
 		self.absname = absname
 				
 	def action(self, text=None):
-		gnome.url_show(self.absname)
+		try:
+			gnome.url_show(gnomevfs.escape_host_and_path_string(self.absname))
+		except Exception, e:
+			if not spawn_async([gnomevfs.get_local_path_from_uri(self.absname)]):
+				gnome.url_show(gnomevfs.escape_host_and_path_string(dirname(self.absname)))
+
 	
 	def is_valid(self, text=None):
 		return exists(self.absname)
@@ -56,7 +62,7 @@ class FolderMatch(deskbar.Match.Match):
 		self.absname = absname
 		
 	def action(self, text=None):
-		gnome.url_show(self.absname)
+		gnome.url_show(gnomevfs.escape_host_and_path_string(self.absname))
 	
 	def is_valid(self, text=None):
 		return exists(self.absname)
@@ -76,7 +82,7 @@ class GtkBookmarkMatch(deskbar.Match.Match):
 		self.path = path
 		
 	def action(self, text=None):
-		gobject.spawn_async(["nautilus", self.path], flags=gobject.SPAWN_SEARCH_PATH)
+		spawn_async(["nautilus", self.path])
 	
 	def is_valid(self, text=None):
 		return exists(self.path)
@@ -96,8 +102,11 @@ class VolumeMatch (deskbar.Match.Match):
 		self.drive = drive
 	
 	def action(self, text=None):
-		gobject.spawn_async(["nautilus", self.drive], flags=gobject.SPAWN_SEARCH_PATH)
-	
+		if self.drive == None:
+			spawn_async(["nautilus"])
+		else:
+			spawn_async(["nautilus", self.drive])
+
 	def is_valid(self, text=None):
 		return exists(self.drive)
 		
