@@ -1,33 +1,40 @@
 import os, ConfigParser, cgi, re
 import glob
-from os.path import join, isfile, abspath, splitext, expanduser, exists
+from os.path import join, isfile, abspath, splitext, expanduser, exists, isdir
 from gettext import gettext as _
-
+from deskbar.defs import VERSION
 import gobject
 import gtk
 import deskbar, deskbar.Indexer, deskbar.Utils
-import deskbar.Handler, deskbar.gnomedesktop
-from deskbar.Utils import get_xdg_data_dirs
+import deskbar.Handler, deskbar.Match, deskbar.gnomedesktop
+from deskbar.Utils import get_xdg_data_dirs, is_program_in_path, spawn_async
 
 HANDLERS = {
 	"ProgramsHandler" : {
 		"name": _("Programs"),
 		"description": _("Launch a program by its name and/or description"),
+		"version": VERSION,
 	},
 	"GnomeDictHandler" : {
 		"name": _("Dictionary"),
 		"description": _("Look up word definitions in the dictionary"),
+		"version": VERSION,
 	},
 	"GnomeSearchHandler" : {
 		"name": _("Files and Folders Search"),
 		"description": _("Find files and folders by searching for a name pattern"),
+		"version": VERSION,
 	},
 	"DevhelpHandler" : {
 		"name": _("Developer Documentation"),
 		"description": _("Search Devhelp for a function name"),
+		"version": VERSION,
 	},
 }
 
+EXACT_MATCH_PRIO = 100
+EXACT_WORD_PRIO = 50
+
 class GenericProgramMatch(deskbar.Match.Match):
 	def __init__(self, backend, use_arg=False, desktop=None, desktop_file=None, **args):
 		deskbar.Match.Match.__init__(self, backend, **args)
@@ -35,6 +42,7 @@ class GenericProgramMatch(deskbar.Match.Match):
 		self.desktop_file = desktop_file
 		self.use_arg = use_arg
 		
+		self._priority = EXACT_WORD_PRIO
 		self._icon = deskbar.Utils.load_icon_for_desktop_icon(self.icon)
 		self._desktop = desktop
 		if desktop == None:
@@ -61,11 +69,15 @@ class GenericProgramMatch(deskbar.Match.Match):
 				args = args + self._args
 			args = args + text.split(" ")
 
-			gobject.spawn_async(args, flags=gobject.SPAWN_SEARCH_PATH)
+			spawn_async(args)
 			# FIXME: This does not launch the App with passed parameters because they are not files..
 			#self._desktop.launch(text.split(" "), deskbar.gnomedesktop.LAUNCH_APPEND_PATHS|deskbar.gnomedesktop.LAUNCH_ONLY_ONE)
 		else:
-			self._desktop.launch([])
+			try:
+				self._desktop.launch([])
+			except Exception, e:
+				#FIXME: Proper dialog here. Also see end of Utils.py
+				print 'Warning:Could not launch .desktop file:', e
 
 	def get_category(self):
 		return "actions"
@@ -121,6 +133,7 @@ class SpecialProgramHandler(deskbar.Handler.Handler):
 		
 	def query(self, qstring):
 		if self._match != None:
+			self._match._priority = get_priority_for_name(qstring, self._match._desktop.get_string("Exec"))
 			return [self._match]
 		else:
 			return []
@@ -160,6 +173,53 @@ class DevhelpHandler(SpecialProgramHandler):
 					icon=desktop.get_string(deskbar.gnomedesktop.KEY_ICON),
 					desktop=desktop,
 					desktop_file=f)
+
+class PathProgramMatch(deskbar.Match.Match):
+	def __init__(self, backend, name=None, use_terminal=False, priority=0, **args):
+		deskbar.Match.Match.__init__(self, backend, name=name, **args)
+		self.use_terminal = use_terminal
+		self._priority = EXACT_MATCH_PRIO
+		
+	def set_with_terminal(self, terminal):
+		self.use_terminal = terminal
+		
+	def get_hash(self, text=None):
+		if not self.use_terminal:
+			return text
+		else:
+			return (text, True)
+		
+	def action(self, text=None):
+		if self.use_terminal:
+			try:
+				prog = subprocess.Popen(
+					text.split(" "),
+					stdout=subprocess.PIPE,
+					stderr=subprocess.STDOUT)
+				
+				zenity = subprocess.Popen(
+					["zenity", "--title="+text,
+						"--window-icon="+join(deskbar.ART_DATA_DIR, "generic.png"),
+						"--width=700",
+						"--height=500",
+						"--text-info"],
+					stdin=prog.stdout)
+	
+				# Reap the processes when they have done
+				gobject.child_watch_add(zenity.pid, lambda pid, code: None)
+				gobject.child_watch_add(prog.pid, lambda pid, code: None)
+				return
+			except:
+				#No zenity, get out of the if, and launch without GUI
+				pass
+		
+		spawn_async(text.split(" "))			
+
+	def get_category(self):
+		return "actions"
+	
+	def get_verb(self):
+		return _("Execute %s") % "<b>%(text)s</b>"
 		
 class ProgramsHandler(deskbar.Handler.Handler):
 	def __init__(self):
@@ -170,8 +230,40 @@ class ProgramsHandler(deskbar.Handler.Handler):
 		self._scan_desktop_files()
 		
 	def query(self, query):
-		return self._indexer.look_up(query)[:deskbar.DEFAULT_RESULTS_PER_HANDLER]
+		result = self.query_path_programs(query)
+		result += self.query_desktop_programs(query)
+		return result
 		
+	def query_path_programs(self, query):
+		args = query.split(" ")
+		match = self._check_program(args[0])
+
+		if match != None:
+			return [match]
+		else:
+			return []
+	
+	def query_desktop_programs(self, query):
+		result = []
+		for match in self._indexer.look_up(query):
+			match._priority = get_priority_for_name(query, match._desktop.get_string("Exec"))
+			result.append(match)
+		return result
+			
+	
+	def on_key_press(self, query, shortcut):
+		if shortcut == gtk.keysyms.t:
+			match = self._check_program(query.split(" ")[0])
+			if match != None:
+				match.set_with_terminal(True)
+				return match
+			
+		return None
+		
+	def _check_program(self, program):
+		if is_program_in_path(program):
+			return PathProgramMatch(self, program)	
+				
 	def _scan_desktop_files(self):
 		for dir in get_xdg_data_dirs():
 			for f in glob.glob(join(dir, "applications", "*.desktop")):
@@ -191,6 +283,12 @@ class ProgramsHandler(deskbar.Handler.Handler):
 								result.get_string(deskbar.gnomedesktop.KEY_COMMENT),
 							), match)
 
+def get_priority_for_name(query, name):
+	if name.split(" ")[0].endswith(query):
+		return EXACT_MATCH_PRIO
+	else:
+		return EXACT_WORD_PRIO
+		
 def parse_desktop_filename(desktop, only_if_visible=True):
 	if desktop[0] == "/" and exists(desktop):
 		return parse_desktop_file(desktop, only_if_visible)
