@@ -68,7 +68,7 @@ class CuemiacUIManager (gobject.GObject) :
 		
 		self.layout = layout
 		
-		self.default_entry_pixbuf = deskbar.Utils.load_icon("deskbar-applet-small.png", width=-1)
+		self.default_entry_pixbuf = deskbar.Utils.load_icon("deskbar-applet-panel-h.png", width=23, height=14)
 		self.clipboard = gtk.clipboard_get (selection="PRIMARY")
 
 		self.entry = CuemiacEntry (self.default_entry_pixbuf)
@@ -187,11 +187,6 @@ class CuemiacUIManager (gobject.GObject) :
 		self.model.append (matches)
 		self._update_entry_icon ()
 		
-		# The UI might have trouble drawing the view
-		# correctly when it is put in a gtk.ScrollView
-		# issue a resize of the view to help redrawing correctly
-		self.cview.queue_resize () # FIXME: Is this really a gtk bug in Ubuntu Dapper?
-		
 		self.layout.on_matches_added (self)
 		
 	def unselect_all (self):
@@ -317,10 +312,10 @@ class CuemiacUIManager (gobject.GObject) :
 		iter = None
 		if path != None:
 			iter = self.model.get_iter (path)
-			
-		if iter is None:
+		
+		if iter is None or self.model.iter_has_child(iter):
 			if self._orient in [gnomeapplet.ORIENT_DOWN, gnomeapplet.ORIENT_LEFT, gnomeapplet.ORIENT_RIGHT]:
-				# No selection, select top element # FIXME do this
+				# No selection, select top element
 				iter = self.model.get_iter_first()
 				while iter != None and ((not self.model.iter_has_child(iter)) or (not self.cview.row_expanded(self.model.get_path(iter)))):
 					iter = self.model.iter_next(iter)
@@ -329,8 +324,9 @@ class CuemiacUIManager (gobject.GObject) :
 
 			else:
 				# We are on a bottom panel - select the bottom element in the list 
-				#FIXME: Should we iterate backwards up the list if the hit is a category?
-				iter = self.model.get_iter (self.cview.last_visible_path())
+				last_path = self.cview.last_visible_path()
+				if last_path:
+					iter = self.model.get_iter (last_path)
 
 		if iter is None:
 			return
